package com.zyzf.coffeepreorder.ui.coffee

import android.content.Context
import android.content.res.Configuration
import android.net.Uri
import android.util.Log
import androidx.activity.compose.ManagedActivityResultLauncher
import androidx.activity.compose.rememberLauncherForActivityResult
import androidx.activity.result.PickVisualMediaRequest
import androidx.activity.result.contract.ActivityResultContracts
import androidx.compose.foundation.border
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.heightIn
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Add
import androidx.compose.material.icons.outlined.Create
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.FloatingActionButton
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.ModalBottomSheet
import androidx.compose.material3.OutlinedIconButton
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Scaffold
import androidx.compose.material3.SheetState
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.material3.rememberModalBottomSheetState
import androidx.compose.runtime.Composable
import androidx.compose.runtime.MutableState
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableDoubleStateOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.lifecycle.viewmodel.compose.viewModel
import coil.compose.AsyncImage
import coil.request.ImageRequest
import com.zyzf.coffeepreorder.R
import com.zyzf.coffeepreorder.database.model.Coffee
import com.zyzf.coffeepreorder.ui.AppViewModelProvider
import com.zyzf.coffeepreorder.ui.theme.CoffeePreorderTheme
import kotlinx.coroutines.launch

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun CoffeeList(
    viewModel: CoffeeListViewModel = viewModel(factory = AppViewModelProvider.Factory)
) {
    val coroutineScope = rememberCoroutineScope()
    val coffeeListUiState = viewModel.coffeeListUiState.collectAsState()
    val sheetState = rememberModalBottomSheetState()
    val openDialog = remember { mutableStateOf(false) }
    val photoPicker = rememberLauncherForActivityResult(
        contract = ActivityResultContracts.PickVisualMedia()
    ) {
        if (it != null) {
            Log.d("PhotoPicker", "Selected URI: $it")
            viewModel.imageUri = it
        } else {
            Log.d("PhotoPicker", "No media selected")
        }
    }
    Scaffold(
        topBar = {},
        floatingActionButton = {
            FloatingActionButton(
                onClick = {
                    coroutineScope.launch {
                        viewModel.cleanCurrentCoffee()
                    }
                },
                Modifier
                    .padding(all = 20.dp)
            ) {
                Icon(
                    imageVector = Icons.Filled.Add,
                    contentDescription = "Add",
                    modifier = Modifier.size(20.dp)
                )
            }
        }
    ) { innerPadding ->
        coffeeListUiState?.value?.coffeeList?.let {
            CoffeeList(
                modifier = Modifier
                    .padding(innerPadding)
                    .fillMaxSize(),
                coffeeList = it,
                onAddToCartClick = { coffeeUid: Int ->
                    coroutineScope.launch {
                        viewModel.addCoffeeToCart(coffeeUid = coffeeUid)
                    }
                },
                onEditClick = { coffee: Coffee ->
                    coroutineScope.launch {
                        viewModel.editCoffee(coffee = coffee)
                    }
                }
            )
        }
        AddEditModalBottomSheet(
            coffee = viewModel.currentCoffee,
            sheetState = sheetState,
            openDialog = openDialog,
            onAddClick = { coffee: Coffee, context: Context ->
                coroutineScope.launch {
                    viewModel.createCoffee(coffee, context)
                }
            },
            onEditClick = { coffee: Coffee, context: Context ->
                coroutineScope.launch {
                    viewModel.editCoffee(coffee, context)
                }
            },
            onDeleteClick = { coffee: Coffee ->
                coroutineScope.launch {
                    viewModel.deleteCoffee(coffee)
                }
            },
            photoPicker = photoPicker,
            imageUri = viewModel.imageUri
        )
    }
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
private fun AddEditModalBottomSheet(
    coffee: Coffee,
    sheetState: SheetState,
    openDialog: MutableState<Boolean>,
    onAddClick: (coffee: Coffee, context: Context) -> Unit,
    onEditClick: (coffee: Coffee, context: Context) -> Unit,
    onDeleteClick: (coffee: Coffee) -> Unit,
    photoPicker: ManagedActivityResultLauncher<PickVisualMediaRequest, Uri?>,
    imageUri: Any?,
    modifier: Modifier = Modifier
) {
    var name by remember { mutableStateOf(coffee.name) }
    var cost by remember { mutableDoubleStateOf(coffee.cost) }
    var ingredients by remember { mutableStateOf(coffee.ingredients) }
    val context = LocalContext.current
    if (openDialog.value) {
        ModalBottomSheet(
            onDismissRequest = { openDialog.value = false },
            sheetState = sheetState
        ) {
            Column(modifier = Modifier
                .fillMaxWidth()
                .padding(all = 10.dp),
                horizontalAlignment = Alignment.CenterHorizontally) {
                OutlinedTextField(modifier = Modifier.fillMaxWidth(),
                    value = name, onValueChange = {name = it},
                    label = {
                        Text(stringResource(id = R.string.coffee_name))
                    },
                    keyboardOptions = KeyboardOptions(keyboardType = KeyboardType.Text)
                )
                OutlinedTextField(modifier = Modifier.fillMaxWidth(),
                    value = cost.toString(), onValueChange = {cost = it.toDouble()},
                    label = {
                        Text(stringResource(id = R.string.coffee_cost))
                    },
                    keyboardOptions = KeyboardOptions(keyboardType = KeyboardType.Decimal)
                )
                OutlinedTextField(modifier = Modifier
                    .fillMaxWidth()
                    .padding(bottom = 10.dp),
                    value = ingredients, onValueChange = {ingredients = it},
                    label = {
                        Text(stringResource(id = R.string.coffee_ingredients))
                    },
                    keyboardOptions = KeyboardOptions(keyboardType = KeyboardType.Text)
                )
                AsyncImage(
                    modifier = Modifier
                        .size(100.dp)
                        .clip(RoundedCornerShape(50.dp))
                        .border(
                            2.dp,
                            MaterialTheme.colorScheme.outline,
                            shape = RoundedCornerShape(50.dp)
                        )
                        .clickable {
                            photoPicker.launch(
                                PickVisualMediaRequest(
                                    ActivityResultContracts.PickVisualMedia.ImageOnly
                                )
                            )
                        }
                        .border(
                            2.dp,
                            MaterialTheme.colorScheme.outlineVariant,
                            shape = RoundedCornerShape(50.dp)
                        ),
                    contentDescription = "Кофе",
                    error = painterResource(R.drawable.ic_broken_image),
                    placeholder = painterResource(R.drawable.loading_img),
                    contentScale = ContentScale.Crop,
                    model = ImageRequest.Builder(LocalContext.current)
                        .data(if (coffee.name != "") "https://zyzf.space/s/zXgFRTmbR4KMxMH/download?path=&files=coffee_image_" + coffee.uid +".png" else imageUri)
                        .crossfade(enable = true)
                        .build(),
                )
                Row(modifier = Modifier.fillMaxWidth(), horizontalArrangement = Arrangement.Center) {
                    if (coffee.uid == 0) {
                        Button(onClick = {
                            onAddClick(coffee, context)
                            openDialog.value = false
                        }, modifier = Modifier.padding(0.dp, 10.dp, 0.dp, 30.dp)) {
                            Text("Добавить")
                        }
                    } else {
                        Button(onClick = {
                            onEditClick(coffee, context)
                            openDialog.value = false
                        }, modifier = Modifier.padding(0.dp, 10.dp, 0.dp, 30.dp)) {
                            Text("Изменить")
                        }
                        Spacer(modifier = Modifier.padding(all = 20.dp))
                        Button(onClick = {
                            onDeleteClick(coffee)
                            openDialog.value = false
                        }, modifier = Modifier.padding(0.dp, 10.dp, 0.dp, 30.dp)) {
                            Text("Удалить")
                        }
                    }
                }
            }
        }
    }
}

@Composable
private fun CoffeeList(
    modifier: Modifier = Modifier,
    coffeeList: List<Coffee>,
    onAddToCartClick: (coffeeUid: Int) -> Unit,
    onEditClick: (coffee: Coffee) -> Unit
) {
    LazyColumn(
        horizontalAlignment = Alignment.CenterHorizontally) {
        items(items = coffeeList, key = { it.uid!! }) { coffee ->
            CoffeeListItem(coffee = coffee, onAddToCartClick = onAddToCartClick, onEditClick = onEditClick)
        }
    }
}

@Composable
private fun CoffeeListItem(
    coffee: Coffee,
    modifier: Modifier = Modifier,
    onAddToCartClick: (coffeeUid: Int) -> Unit,
    onEditClick: (coffee: Coffee) -> Unit
) {
    Row(modifier = Modifier
        .fillMaxWidth()
        .heightIn(max = 140.dp)
        .padding(bottom = 10.dp, top = 10.dp),
        horizontalArrangement = Arrangement.SpaceAround) {

        AsyncImage(
            model = ImageRequest.Builder(context = LocalContext.current).data("https://zyzf.space/s/zXgFRTmbR4KMxMH/download?path=&files=coffee_image_" + coffee.uid +".png")
                .crossfade(true).build(),
            error = painterResource(R.drawable.ic_broken_image),
            placeholder = painterResource(R.drawable.loading_img),
            contentDescription = "Кофе",
            contentScale = ContentScale.Crop,
            modifier = Modifier
                .size(100.dp)
                .clip(RoundedCornerShape(50.dp))
        )

        Column(
            Modifier
                .weight(2f)
                .padding(start = 20.dp)) {
            Text(text = coffee.name, fontSize = 25.sp)
            Text(text = String.format("%.2f", coffee.cost), fontSize = 20.sp)
            Text(text = coffee.ingredients, fontSize = 15.sp)
            Row(
                Modifier
                    .fillMaxWidth()
                    .padding(top = 5.dp)) {
                Button(
                    onClick = {
                        coffee.uid?.let { onAddToCartClick(it) }
                    },
                    shape = CircleShape,
                    modifier = Modifier.fillMaxWidth(fraction = 0.75f)
                ) {
                    Icon(
                        imageVector = Icons.Default.Add,
                        contentDescription = "Favorite",
                        modifier = Modifier.size(20.dp)
                    )
                    Spacer(Modifier.size(ButtonDefaults.IconSpacing))
                    Text(text = "В корзину")
                }
                OutlinedIconButton(
                    onClick = {
                        onEditClick(coffee)
                    },
                    Modifier
                        .padding(start = 10.dp)
                        .clip(CircleShape)) {
                    Icon(
                        imageVector = Icons.Outlined.Create,
                        contentDescription = "Favorite",
                        modifier = Modifier.size(20.dp)
                    )
                }
            }
        }
    }
}

@Preview(name = "Light Mode", showBackground = true, uiMode = Configuration.UI_MODE_NIGHT_NO)
@Preview(name = "Dark Mode", showBackground = true, uiMode = Configuration.UI_MODE_NIGHT_YES)
@Composable
fun CoffeeListPreview() {
    CoffeePreorderTheme {
        Surface(
            color = MaterialTheme.colorScheme.background
        ) {
            CoffeeList(
                coffeeList = (1..20).map { i -> Coffee.getCoffee(i) },
                onAddToCartClick = {},
                onEditClick = {}
            )
        }
    }
}